﻿using Microsoft.Extensions.Logging;
using PizzeriaContracts.BindingModels;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Models;

namespace SushiBusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        private readonly ISushiStorage _SushiStorage;
        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage, ISushiStorage SushiStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
            _SushiStorage = SushiStorage;
        }
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName: {ShopName}. Id: {Id}",
                model?.ShopName, model?.Id);
            var list = model == null ? _shopStorage.GetFullList() :
                _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ShopName: {ShopName}. Id: {Id}",
                model.ShopName, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
            return element;
        }
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
            }
            _logger.LogInformation("Shop. ShopName:{0}.Address:{1}. Id: {2}",
                model.ShopName, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = model.ShopName
            });
            if (element != null && element.Id != model.Id && element.ShopName == model.ShopName)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
        public bool MakeSupply(SupplyBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentException("Количество изделий должно быть больше 0");
            }
            var shop = _shopStorage.GetElement(new ShopSearchModel
            {
                Id = model.ShopId
            });
            if (shop == null)
            {
                throw new ArgumentException("Магазина не существует");
            }
            if (shop.ShopSushi.ContainsKey(model.SushiId))
            {
                var oldValue = shop.ShopSushi[model.SushiId];
                oldValue.Item2 += model.Count;
                shop.ShopSushi[model.SushiId] = oldValue;
            }
            else
            {
                var sushi = _SushiStorage.GetElement(new SushiSearchModel
                {
                    Id = model.SushiId
                });
                if (sushi == null)
                {
                    throw new ArgumentException($"Поставка: Товар с id:{model.SushiId} не найденн");
                }
                shop.ShopSushi.Add(model.SushiId, (sushi, model.Count));
            }
            return true;
        }
    }
}
